@@ -65,6 +65,7 @@ if idx == 1:
 '''
 
 SCRIPT_PARENT = '''
+import os
 import subprocess
 import sys
 
@@ -74,7 +75,7 @@ def pytest_generate_tests(metafunc):
 
 def test_foo(idx):
     out, err = subprocess.Popen(
-        [sys.executable, 'child_script.py', str(idx)],
+        [sys.executable, os.path.join(os.path.dirname(__file__), 'child_script.py'), str(idx)],
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE).communicate()
 
@@ -149,7 +150,7 @@ def test_fail():
 '''
 
 CHILD_SCRIPT_RESULT = '[56] * 100%'
-PARENT_SCRIPT_RESULT = '8 * 100%'
+PARENT_SCRIPT_RESULT = '9 * 100%'
 DEST_DIR = 'cov_dest'
 REPORT_NAME = 'cov.xml'
 
@@ -168,6 +169,7 @@ def prop(request):
         code2=SCRIPT2,
         conf=request.param[0],
         fullconf='[run]\n%s\n' % request.param[0],
+        prefixedfullconf='[coverage:run]\n%s\n' % request.param[0],
         args=request.param[1].split(),
         result=request.param[2],
         result2=request.param[3],
@@ -227,7 +229,7 @@ def test_annotate_output_dir(testdir):
     assert result.ret == 0
 
 
-def test_html_output_dir(testdir,prop):
+def test_html_output_dir(testdir, prop):
     script = testdir.makepyfile(SCRIPT)
 
     result = testdir.runpytest('-v',
@@ -271,10 +273,7 @@ def test_term_output_dir(testdir):
                                '--cov-report=term:' + DEST_DIR,
                                script)
 
-    # backport of argparse to py26 doesn't display ArgumentTypeError message
     result.stderr.fnmatch_lines([
-        '*argument --cov-report: *',
-    ] if tuple(sys.version_info[:2]) == (2, 6) else [
         '*argument --cov-report: output specifier not supported for: "term:%s"*' % DEST_DIR,
     ])
     assert result.ret != 0
@@ -288,10 +287,7 @@ def test_term_missing_output_dir(testdir):
                                '--cov-report=term-missing:' + DEST_DIR,
                                script)
 
-    # backport of argparse to py26 doesn't display ArgumentTypeError message
     result.stderr.fnmatch_lines([
-        '*argument --cov-report: *',
-    ] if tuple(sys.version_info[:2]) == (2, 6) else [
         '*argument --cov-report: output specifier not supported for: '
         '"term-missing:%s"*' % DEST_DIR,
     ])
@@ -400,6 +396,79 @@ def test_central_coveragerc(testdir, prop):
     assert result.ret == 0
 
 
+@xdist
+def test_central_with_path_aliasing(testdir, monkeypatch, opts, prop):
+    mod1 = testdir.mkdir('src').join('mod.py')
+    mod1.write(SCRIPT)
+    mod2 = testdir.mkdir('aliased').join('mod.py')
+    mod2.write(SCRIPT)
+    script = testdir.makepyfile('''
+from mod import *
+''')
+    testdir.tmpdir.join('setup.cfg').write("""
+[coverage:paths]
+source =
+    src
+    aliased
+[coverage:run]
+source = mod
+parallel = true
+%s
+""" % prop.conf)
+
+    monkeypatch.setitem(os.environ, 'PYTHONPATH', os.pathsep.join([os.environ.get('PYTHONPATH', ''), 'aliased']))
+    result = testdir.runpytest('-v', '-s',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               script, *opts.split()+prop.args)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'src[\\/]mod* %s *' % prop.result,
+        '*10 passed*',
+    ])
+
+    # single-module coverage report
+    assert all(not line.startswith('TOTAL ') for line in result.stdout.lines[-4:])
+
+    assert result.ret == 0
+
+
+def test_subprocess_with_path_aliasing(testdir, monkeypatch):
+    src = testdir.mkdir('src')
+    src.join('parent_script.py').write(SCRIPT_PARENT)
+    src.join('child_script.py').write(SCRIPT_CHILD)
+    aliased = testdir.mkdir('aliased')
+    parent_script = aliased.join('parent_script.py')
+    parent_script.write(SCRIPT_PARENT)
+    aliased.join('child_script.py').write(SCRIPT_CHILD)
+
+    testdir.tmpdir.join('.coveragerc').write("""
+[paths]
+source =
+    src
+    aliased
+[run]
+source =
+    parent_script
+    child_script
+parallel = true
+""")
+
+    monkeypatch.setitem(os.environ, 'PYTHONPATH', os.pathsep.join([os.environ.get('PYTHONPATH',''), 'aliased']))
+    result = testdir.runpytest('-v',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               parent_script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'src[\\/]child_script* %s*' % CHILD_SCRIPT_RESULT,
+        'src[\\/]parent_script* %s*' % PARENT_SCRIPT_RESULT,
+    ])
+    assert result.ret == 0
+
+
 def test_show_missing_coveragerc(testdir, prop):
     script = testdir.makepyfile(prop.code)
     testdir.tmpdir.join('.coveragerc').write("""
@@ -453,7 +522,7 @@ def test_no_cov(testdir):
                                script)
     result.stdout.fnmatch_lines_random([
         'WARNING: Coverage disabled via --no-cov switch!',
-        '*1 warnings*' if pytest.__version__.startswith('2.7') else 'WCOV-U1*Coverage disabled via --no-cov switch!',
+        '*Coverage disabled via --no-cov switch!',
     ])
 
 
@@ -524,7 +593,14 @@ def test_dist_not_collocated(testdir, prop):
     script = testdir.makepyfile(prop.code)
     dir1 = testdir.mkdir('dir1')
     dir2 = testdir.mkdir('dir2')
-    testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
+    testdir.tmpdir.join('.coveragerc').write('''
+[run]
+%s
+[paths]
+source =
+    .
+    dir1
+    dir2''' % prop.conf)
 
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
@@ -545,6 +621,39 @@ def test_dist_not_collocated(testdir, prop):
     assert result.ret == 0
 
 
+def test_dist_not_collocated_coveragerc_source(testdir, prop):
+    script = testdir.makepyfile(prop.code)
+    dir1 = testdir.mkdir('dir1')
+    dir2 = testdir.mkdir('dir2')
+    testdir.tmpdir.join('.coveragerc').write('''
+[run]
+%s
+source = %s
+[paths]
+source =
+    .
+    dir1
+    dir2''' % (prop.conf, script.dirpath()))
+
+    result = testdir.runpytest('-v',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               '--dist=load',
+                               '--tx=popen//chdir=%s' % dir1,
+                               '--tx=popen//chdir=%s' % dir2,
+                               '--rsyncdir=%s' % script.basename,
+                               '--rsyncdir=.coveragerc',
+                               '--max-slave-restart=0', '-s',
+                               script, *prop.args)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_dist_not_collocated* %s *' % prop.result,
+        '*10 passed*'
+    ])
+    assert result.ret == 0
+
+
 def test_central_subprocess(testdir):
     scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
                                  child_script=SCRIPT_CHILD)
@@ -581,8 +690,8 @@ parallel = true
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'child_script* %s*' % CHILD_SCRIPT_RESULT,
-        'parent_script* 100%*',
+        '*child_script* %s*' % CHILD_SCRIPT_RESULT,
+        '*parent_script* 100%*',
     ])
     assert result.ret == 0
 
@@ -622,8 +731,6 @@ def test_foo():
     testdir.makefile('', coveragerc="""
 [run]
 parallel = true
-omit =
-    */__init__.py
 """)
     result = testdir.runpytest('-v',
                                '--cov-config=coveragerc',
@@ -666,15 +773,21 @@ def test_dist_subprocess_not_collocated(testdir, tmpdir):
 
     dir1 = tmpdir.mkdir('dir1')
     dir2 = tmpdir.mkdir('dir2')
-
+    testdir.tmpdir.join('.coveragerc').write('''
+[paths]
+source =
+    %s
+    */dir1
+    */dir2
+''' % scripts.dirpath())
     result = testdir.runpytest('-v',
                                '--cov=%s' % scripts.dirpath(),
-                               '--cov-report=term-missing',
                                '--dist=load',
                                '--tx=popen//chdir=%s' % dir1,
                                '--tx=popen//chdir=%s' % dir2,
                                '--rsyncdir=%s' % child_script,
                                '--rsyncdir=%s' % parent_script,
+                               '--rsyncdir=.coveragerc',
                                '--max-slave-restart=0',
                                parent_script)
 
@@ -688,7 +801,10 @@ def test_dist_subprocess_not_collocated(testdir, tmpdir):
 
 def test_invalid_coverage_source(testdir):
     script = testdir.makepyfile(SCRIPT)
-
+    testdir.makeini("""
+        [pytest]
+        console_output_style=classic
+    """)
     result = testdir.runpytest('-v',
                                '--cov=non_existent_module',
                                '--cov-report=term-missing',
@@ -700,16 +816,10 @@ def test_invalid_coverage_source(testdir):
     result.stderr.fnmatch_lines([
         'Coverage.py warning: No data was collected.*'
     ])
-
-    if StrictVersion(coverage.__version__) <= StrictVersion("3.8"):
-        # older `coverage report` doesn't error on missing imports
-        assert result.ret == 0
-    else:
-        # newer `coverage report` errors on missing imports
-        result.stderr.fnmatch_lines([
-            'ERROR: Failed to generate report: No data to report.',
-        ])
-        assert result.ret != 0
+    result.stdout.fnmatch_lines([
+        '*Failed to generate report: No data to report.',
+    ])
+    assert result.ret == 0
 
     matching_lines = [line for line in result.outlines if '%' in line]
     assert not matching_lines
@@ -725,7 +835,7 @@ def test_dist_missing_data(testdir):
         exe = os.path.join(venv_path, 'bin', 'python')
     subprocess.check_call([
         exe,
-        '-mpip' if sys.version_info >= (2, 7) else '-mpip.__main__',
+        '-mpip',
         'install',
         'py==%s' % py.__version__,
         'pytest==%s' % pytest.__version__
@@ -803,6 +913,35 @@ def test_run_target():
     assert result.ret == 0
 
 
+def test_multiprocessing_subprocess_no_source(testdir):
+    py.test.importorskip('multiprocessing.util')
+
+    script = testdir.makepyfile('''
+import multiprocessing
+
+def target_fn():
+    a = True
+    return a
+
+def test_run_target():
+    p = multiprocessing.Process(target=target_fn)
+    p.start()
+    p.join()
+''')
+
+    result = testdir.runpytest('-v',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_multiprocessing_subprocess* 8 * 100%*',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
 @pytest.mark.skipif('sys.platform == "win32"',
                     reason="multiprocessing don't support clean process temination on Windows")
 def test_multiprocessing_subprocess_with_terminate(testdir):
@@ -824,6 +963,7 @@ def target_fn():
 def test_run_target():
     p = multiprocessing.Process(target=target_fn)
     p.start()
+    time.sleep(0.5)
     event.wait(1)
     p.terminate()
     p.join()
@@ -836,7 +976,7 @@ def test_run_target():
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'test_multiprocessing_subprocess* 15 * 100%*',
+        'test_multiprocessing_subprocess* 16 * 100%*',
         '*1 passed*'
     ])
     assert result.ret == 0
@@ -910,6 +1050,46 @@ def test_cover_conftest_dist(testdir):
     result.stdout.fnmatch_lines([CONF_RESULT])
 
 
+def test_no_cover_marker(testdir):
+    testdir.makepyfile(mod=MODULE)
+    script = testdir.makepyfile('''
+import pytest
+import mod
+import subprocess
+import sys
+
+@pytest.mark.no_cover
+def test_basic():
+    mod.func()
+    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])    
+''')
+    result = testdir.runpytest('-v', '-ra', '--strict',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines(['mod* 2 * 1 * 50% * 2'])
+
+
+def test_no_cover_fixture(testdir):
+    testdir.makepyfile(mod=MODULE)
+    script = testdir.makepyfile('''
+import mod
+import subprocess
+import sys
+
+def test_basic(no_cover):
+    mod.func()
+    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])    
+''')
+    result = testdir.runpytest('-v', '-ra', '--strict',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines(['mod* 2 * 1 * 50% * 2'])
+
+
 COVERAGERC = '''
 [report]
 # Regexes for lines to exclude from consideration
